CON
  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000
  block = $0E
  backspace = $C8
  delete = $C9
  space = $20
  enter = $0D
  pageup = $C6
  pagedown = $C7
  null = $BC
  uparrow = $04
  downarrow = $05
  upkey = $C2
  downkey = $C3
  tvbackspace = $08
  line = $7C
  thickline = $7F
  clearscreen = $00
  powerkey = $CD


OBJ
  KB: "Keyboard"
  TV: "TV_Text-WP"

VAR
  byte x[4000]        'The x-value on screen for each character value
  byte y[4000]        'The y-value on screen for each character value
  byte char[4000]     'The character to display for each character value
  word lineend[120]
  byte linenum
  byte InputKey       'The key that is input via the keyboard
  byte ydisplay
  byte offset
  byte firsttime
  byte tvvalue
  byte cursorok
  byte lastline
  word i
  word j
  word k
  word l
  word m              'Miscellaneous variables used in loops

PUB Start
  KB.Startx(26, 27, %0_000_000, %01_01000)
  TV.Start(12)
  WaitCnt(50_000_000 + cnt)
  Repeat                                                'This loop checks to see if the keyboard
    If KB.Present                                       'is present, and tells the user to plug in
        x[0] := 0                                       'the keyboard if not
        y[0] := 0
        i := 0                                                'Start the program at x=0, y=0, char=0, page 1
        bytefill(@char, null, 4000)                           'Fill character buffer with 'null' characters
        lineend[0] := 0
        linenum := 0
        firsttime := 0
        offset := 0
        TV.setxy(0,12)
        TV.str(string("WELCOME            "))
        TV.setxy(0,0)
        DrawScrollbar(1)
        KeyboardPlayback
    Else
      TV.setxy(0,12)
      TV.str(string("PLUG IN KEYBOARD"))

PRI KeyboardPlayback
  Repeat
    InputKey := KB.GetKey                               'Pause program until keyboard input recieved
    InputRoutine
    If i == 65535
      i := 0  'Prevent bug with delete key
    If x[i] == 37 and cursorok == 0                                      'If end of line reached, go on to new line
      lineend[linenum] := i
      linenum += 1
      i += 1
      j := i - 1
      ydisplay := y[j] - offset
      TV.setxy(37,ydisplay)
      TV.out(space)
      x[i] := 0
      y[i] := y[j] + 1
      ydisplay := y[i] - offset
      TV.setxy(x[i],ydisplay)
      TV.out(block)
    ElseIf cursorok == 1
      cursorok := 0
    tvvalue := 12 + offset
    If y[i] == tvvalue
      MoveTextDown(y[i])
    If y[i] < offset
      MoveTextUp(y[i])
    TV.setxy(33, 12)                                    'The following code block displays x and y
    TV.dec(x[i])                                        'values in lower-right corner
    TV.out(",")
    TV.dec(y[i])
    'TV.out(",")                                        'Uncomment this code to display variable values for debugging
    'TV.dec(i)
    'TV.out(",")
    'TV.dec(linenum)
    'TV.out(",")
    'TV.dec(offset)
    TV.str(string("  "))
    ydisplay := y[i] - offset
    TV.setxy(x[i], ydisplay)

PRI InputRoutine
  Case InputKey
    backspace:                  'If backspace pressed, go back a space
      j := i - 1
      k := linenum * 38
      k -= 1
      If i <> 0
        If k <> j
          TV.out(space)
          i -= 1
          lineend[linenum] := i
          char[i] := null
          linenum := y[i]
          ydisplay := y[i] - offset
          TV.setxy(x[i], ydisplay)
          TV.out(block)
          TV.out(space)
          DrawScrollbar(linenum)
        Else
          TV.out(space)
          linenum -= 1
          i := lineend[linenum]
          ydisplay := y[i] - offset
          TV.setxy(x[i], ydisplay)
          TV.out(block)
          DrawScrollbar(linenum)
      If x[i] == 37
        cursorok := 1
    delete:                     'If delete key pressed, go back a space
      j := i - 1
      k := linenum * 38
      k -= 1
      If i <> 0
        If k <> j
          TV.out(space)
          i -= 1
          lineend[linenum] := i
          char[i] := null
          linenum := y[i]
          ydisplay := y[i] - offset
          TV.setxy(x[i], ydisplay)
          TV.out(block)
          TV.out(space)
          DrawScrollbar(linenum)
        Else
          TV.out(space)
          linenum -= 1
          i := lineend[linenum]
          ydisplay := y[i] - offset
          TV.setxy(x[i], ydisplay)
          TV.out(block)
          DrawScrollbar(linenum)
      If x[i] == 37
        cursorok := 1
    enter:                      'If enter key pressed, go down a line
      TV.out(space)
      lineend[linenum] := i
      linenum += 1
      j := i
      i := (linenum * 38)
      x[i] := 0
      y[i] := y[j] + 1
      ydisplay := y[i] - offset
      TV.setxy(x[i],ydisplay)
      TV.out(block)
      'DrawScrollbar(linenum)
    upkey:
      MoveCursorUp
      DrawScrollbar(linenum)
    downkey:
      MoveCursorDown
      DrawScrollbar(linenum)
    powerkey:
      DrawScrollbar(linenum)

    OTHER:                      'If normal key pressed, print character
      ydisplay := y[i] - offset
      TV.setxy(x[i], ydisplay)
      char[i] := InputKey
      TV.out(char[i])
      TV.out(block)
      j := i
      i += 1
      x[i] := x[j] + 1
      y[i] := y[j]
      lineend[linenum] := i
      DrawScrollbar(linenum)

PRI MoveCursorUp
  j := linenum - 1
  k := lineend[j]
  'repeat m from 0 to linenum
  '  l += lineend[m]
  If y[i] > 0
    linenum := j
    i := k
    TV.out(space)
    i := lineend[linenum]
    ydisplay := y[i] - offset
    TV.setxy(x[i], ydisplay)
    TV.out(block)
    If x[i] == 37
      cursorok := 1
  Else
    ydisplay := y[i] - offset
    TV.setxy(x[i], ydisplay)
    If x[i] == 37
      cursorok := 1

PRI MoveCursorDown
  j := y[i] + 1
  k := lineend[j]
  'repeat m from 0 to lastline
  '  l += lineend[m]
  'l += 1
  If k > 0
    i := k
    linenum := y[i]
    TV.out(space)
    ydisplay := y[i] - offset
    TV.setxy(x[i], ydisplay)
    TV.out(block)
    If x[i] == 37
      cursorok := 1

PRI MoveTextDown (linenumber)
  TV.out(clearscreen)
  linenum := linenumber
  offset += 1
  k := offset - 1
  k := lineend[k]
  k += 1
  'j := offset + 10
  'j := lineend[j]
  'j -= 1
  Repeat m from k to i
    ydisplay := y[m] - offset
    TV.setxy(x[m], ydisplay)
    If char[m] <> null
      TV.out(char[m])
  ydisplay := y[i] - offset
  TV.setxy(x[i], ydisplay)
  TV.out(block)
  DrawScrollbar(linenum)

PRI MoveTextUp (linenumber)
  TV.out(clearscreen)
  offset -= 1
  linenum:= linenumber
  j := offset * 38
  l := offset + 11
  repeat m from j to lineend[l]
    ydisplay := y[m] - offset
    TV.setxy(x[m], ydisplay)
    If char[m] <> null
      TV.out(char[m])
  ydisplay := y[i] - offset
  TV.setxy(x[i], ydisplay)
  TV.out(block)
  DrawScrollbar(linenum)

PRI DrawScrollbar (linenumber)
  TV.setxy(39,0)
  TV.out(uparrow)
  TV.setxy(39,12)
  TV.out(downarrow)
  repeat m from 1 to 11
    TV.setxy(39,m)
    TV.out(line)
  m := 0
  k := 0
  l := 0
  repeat until m == 1
    If lineend[k] == 0
      l := k + 1
      If lineend[l] == 0
        m := linenum * 11
        l := m / k
        l += 1
        TV.setxy(39,l)
        TV.out(thickline)
        Return
    k += 1
  ydisplay := y[i] - offset
  TV.setxy(x[i], ydisplay)

